'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MOZ_CHUNKED = undefined;
exports.default = mozXhrRequest;

var _browserHeaders = require('browser-headers');

var _browserHeaders2 = _interopRequireDefault(_browserHeaders);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MOZ_CHUNKED = exports.MOZ_CHUNKED = 'moz-chunked';

function mozXhrRequest(options) {
  var xhr = new XMLHttpRequest();

  function onProgressEvent() {
    options.onRawChunk(new Uint8Array(xhr.response));
  }

  function onLoadEvent() {
    options.onRawComplete({
      statusCode: xhr.status,
      transport: MOZ_CHUNKED,
      raw: xhr
    });
  }

  function onStateChange() {
    if (this.readyState == this.HEADERS_RECEIVED) {
      options.onRawHeaders(new _browserHeaders2.default(this.getAllResponseHeaders()), this.status);
    }
  }
  function onError(err) {
    options.onRawComplete({
      statusCode: 0,
      transport: MOZ_CHUNKED,
      raw: err
    });
  }

  xhr.open(options.method, options.url);
  xhr.responseType = 'moz-chunked-arraybuffer';
  if (options.headers) {
    Object.getOwnPropertyNames(options.headers).forEach(function (k) {
      xhr.setRequestHeader(k, options.headers[k]);
    });
  }
  if (options.credentials === 'include') {
    xhr.withCredentials = true;
  }
  xhr.addEventListener('readystatechange', onStateChange);
  xhr.addEventListener('progress', onProgressEvent);
  xhr.addEventListener('loadend', onLoadEvent);
  xhr.addEventListener('error', onError);
  xhr.send(options.body);
}